@@ -2,9 +2,10 @@
 #
 # scantwo.R
 #
-# copyright (c) 2001-5, Karl W Broman, Johns Hopkins University,
+# copyright (c) 2001-6, Karl W Broman, Johns Hopkins University,
 #            Hao Wu, and Brian Yandell
-# last modified Sep, 2005
+#
+# last modified Jun, 2006
 # first written Nov, 2001
 # Licensed under the GNU General Public License version 2 (June, 1991)
 # 
@@ -16,6 +17,7 @@
 # Part of the R/qtl package
 # Contains: scantwo, scantwo.perm, summary.scantwo
 #           print.summary.scantwo, max.scantwo
+#           scantwo.perm.engine
 #
 ######################################################################
 
@@ -29,21 +31,48 @@
 
 scantwo <-
 function(cross, chr, pheno.col=1,
-         method=c("em","imp","hk","mr","mr-imp","mr-argmax"),
          model=c("normal","binary"),
+         method=c("em","imp","hk","mr","mr-imp","mr-argmax"),
          addcovar=NULL, intcovar=NULL, weights=NULL,
          run.scanone=TRUE, incl.markers=FALSE, maxit=4000, tol=1e-4,
-         verbose=TRUE, n.perm)
+         verbose=TRUE, n.perm, n.node=1)
 {
   method <- match.arg(method)
   model <- match.arg(model)
   
   origcross <- cross
-
+  
   # pull out chromosomes to be scanned
   if(!missing(chr)) cross <- subset(cross,chr=chr)
   if(missing(n.perm)) n.perm <- 0
 
+  if(any(pheno.col < 1 | pheno.col > nphe(cross)))
+    stop("pheno.col values should be between 1 and the no. phenotypes")
+
+  # multiple phenotype for methods other than imp and hk
+  if(length(pheno.col)>1 && n.perm <= 0 &&
+     method!="imp" && method != "hk" ) {
+    n.phe <- length(pheno.col)
+    if(verbose) cat(" -Phenotype 1\n")
+    output <- scantwo(cross, , pheno.col[1], model, method, addcovar,
+                      intcovar, weights, run.scanone, incl.markers, maxit,
+                      tol, verbose, n.perm, n.node)
+    temp <- array(dim=c(nrow(output$lod), ncol(output$lod), n.phe))
+    temp[,,1] <- output$lod
+    output$lod <- temp
+    for(i in 2:n.phe) {
+      if(verbose) cat(" -Phenotype ", i, "\n")
+      temp <- scantwo(cross, , pheno.col[i], model, method, addcovar,
+                      intcovar, weights, run.scanone, incl.markers, maxit,
+                      tol, verbose, n.perm, n.node)
+      output$lod[,,i] <- temp$lod
+      if(!is.null(output$scanoneX))
+        output$scanoneX <- cbind(output$scanoneX, temp$scanoneX)
+    }
+    return(output)
+  }
+
+
   # check phenotypes and covariates; drop individuals with missing values
   # in case of permutation test, only do checks once
   if(n.perm>=0) { 
@@ -55,8 +84,8 @@ function(cross, chr, pheno.col=1,
     n.addcovar <- temp[[5]]
     n.intcovar <- temp[[6]]
   }
-  else {
-    pheno <- cross$pheno[,pheno.col]
+  else {  # come here only in permutation tests
+    pheno <- as.matrix(cross$pheno[,pheno.col])
     if(is.null(addcovar)) n.addcovar <- 0
     else n.addcovar <- ncol(addcovar)
     if(is.null(intcovar)) n.intcovar <- 0
@@ -64,6 +93,7 @@ function(cross, chr, pheno.col=1,
   }
   n.chr <- nchr(cross)
   n.ind <- nind(cross)
+  n.phe <- length(pheno.col)
   type <- class(cross)[1]
   chrtype <- sapply(cross$geno,class)
 
@@ -74,13 +104,14 @@ function(cross, chr, pheno.col=1,
     }
     if(any(chrtype=="X")) {
       sexpgm <- getsex(cross)
-      if(!is.null(sexpgm$sex) || !is.null(sexpgm$pgm)) {
-        cross <- subset(cross,chr = chrtype!="X")
-        n.chr <- nchr(cross)
-        n.ind <- nind(cross)
-        warning("X chromosome is not yet working for binary traits; dropping it.")
-      }
+#      if(!is.null(sexpgm$sex) || !is.null(sexpgm$pgm)) {
+#        cross <- subset(cross,chr = chrtype!="X")
+#        n.chr <- nchr(cross)
+#        n.ind <- nind(cross)
+#        warning("X chromosome is not yet working for binary traits; dropping it.")
+#      }
     }
+
     if(!is.null(weights)) {
       weights <- NULL
       warning("weights ignored for binary traits.")
@@ -91,20 +122,11 @@ function(cross, chr, pheno.col=1,
       stop("Phenotypes must be either 0 or 1.")
   }
 
-  # Problems with EX w/ X chromosome: just use H-K for now.
-  if(model=="normal" && any(chrtype=="X") && method=="em") {
-    sexpgm <- getsex(cross)
-    if(!is.null(sexpgm$sex) || !is.null(sexpgm$pgm)) {
-      warning("EM is not yet working for X chromosomes; using H-K instead.")
-      method <- "hk"
-    }
-  }
-
   # if n.perm specified, do a permutation test
   if(n.perm>0) { 
-    return(scantwo.perm(cross, pheno.col, method, model, addcovar,
+    return(scantwo.perm(cross, pheno.col, model, method, addcovar,
                         intcovar, weights, incl.markers,
-                        maxit, tol, verbose, n.perm))
+                        maxit, tol, verbose, n.perm, n.node))
   }
 
   if(n.perm == 0) { # not in the midst of permutations
@@ -127,12 +149,10 @@ function(cross, chr, pheno.col=1,
 
   if(run.scanone) { # also do scanone
     if(verbose) cat(" --Running scanone\n")
-    temp <- scanone(cross, pheno.col=pheno.col, method=method, model=model,
+    temp <- scanone(cross, pheno.col=pheno.col, model=model, method=method, 
                     addcovar=addcovar, intcovar=intcovar, weights=weights,
                     maxit=maxit, tol=tol, verbose=FALSE)
-    nam <- rownames(temp)
-    out.scanone <- temp[,3]
-    names(out.scanone) <- nam
+    out.scanone <- temp[,-(1:2),drop=FALSE]
     if(verbose) cat(" --Running scantwo\n")
   }
 
@@ -257,7 +277,7 @@ function(cross, chr, pheno.col=1,
       n.pos[i] <- length(keep.pos[[i]])
 
       # Revise X chromosome genotype probabilities or imputations 
-      if(chrtype[i]=="X" && (type=="bc" || type=="f2" || type=="f2ss")) {
+      if(chrtype[i]=="X" && (type=="bc" || type=="f2")) {
         if(method=="imp") 
           cross$geno[[i]]$draws <-
             reviseXdata(type, "full", sexpgm, draws=cross$geno[[i]]$draws)
@@ -281,7 +301,10 @@ function(cross, chr, pheno.col=1,
     wh.col[[i]] <- seq(first.pos[i],by=1,length=n.pos[i])
 
   # initialize the results matrix
-  results <- matrix(0,ncol=sum(n.pos), nrow=sum(n.pos))
+  if(n.phe > 1)
+    results <- array(0,dim=c(sum(n.pos),sum(n.pos), n.phe))
+  else
+    results <- matrix(0,sum(n.pos),sum(n.pos))  
 
   # do the 2-dimensional genome scan
   for(i in 1:n.chr) { # loop over the 1st chromosome
@@ -292,6 +315,7 @@ function(cross, chr, pheno.col=1,
         n.ac <- ifelse(is.null(ac),0,ncol(ac))
         ic <- revisecovar(sexpgm,intcovar)
         n.ic <- ifelse(is.null(ic),0,ncol(ic))
+
       }
       else {
         ac <- addcovar
@@ -299,6 +323,14 @@ function(cross, chr, pheno.col=1,
         ic <- intcovar
         n.ic <- n.intcovar
       }
+      if(i==j && chrtype[i]=="X") {
+        col2drop <- dropXcol(type, sexpgm)
+        n.col2drop <- sum(col2drop)
+      }
+      else {
+        col2drop <- rep(0,n.gen[i]*n.gen[j])
+        n.col2drop <- 0
+      }
 
       # print the current working pair
       if(verbose) cat(paste(" (", names(cross$geno)[i], ",",
@@ -320,15 +352,22 @@ function(cross, chr, pheno.col=1,
                 as.double(ic),
                 as.integer(n.ic),
                 as.double(pheno),
+                as.integer(n.phe),
                 as.double(weights),
-                result=as.double(rep(0,2*n.pos[i]*n.pos[j])),
+                result=as.double(rep(0,2*n.pos[i]*n.pos[j]*n.phe)),
+                as.integer(n.col2drop),
+                as.integer(col2drop),
                 PACKAGE="qtl")
-        z <- array(z$result,dim=c(n.pos[i], n.pos[j], 2)) # rearrange the result 
+        z <- array(z$result,dim=c(n.pos[i], n.pos[j], 2*n.phe)) # rearrange the result 
 
         # update the final result matrix
-        results[wh.col[[i]],wh.col[[j]]] <- z[,,1]
-        if(i != j) results[wh.col[[j]],wh.col[[i]]] <- t(z[,,2])
-        else { # do this just once: do null model and get neg log10 likelihood
+        if(i == j) { # on same chromosome
+          if(n.phe > 1)
+            results[wh.col[[i]],wh.col[[j]],] <- z[,,1:n.phe]
+          else
+            results[wh.col[[i]],wh.col[[j]]] <- z[,,1]
+          
+          # do this just once: do null model and get neg log10 likelihood
           if(i==1) { 
             if(n.ac > 0)
               resid0 <- lm(pheno ~ ac, weights=weights^2)$resid
@@ -337,6 +376,23 @@ function(cross, chr, pheno.col=1,
             sig0 <- sqrt(sum((resid0*weights)^2)/n.ind)
             nllik0 <- -sum(dnorm(resid0,0,sig0/weights,log=TRUE))/log(10)
           }
+
+#          cat("nllik0: ", nllik0, "\n")
+        }
+        else { # on different chromosomes
+          if(n.phe > 1) {
+            # full lod
+            results[wh.col[[i]],wh.col[[j]],] <- z[,,1:n.phe]
+            # epistasis lod - need to reshape the matrix
+            results[wh.col[[j]],wh.col[[i]],] <- array(z[,,(n.phe+1):(2*n.phe)],
+                                                       c(n.pos[j],n.pos[i], n.phe))
+          }
+          else { # only one phenotype, result is a matrix
+            # full lod
+            results[wh.col[[i]],wh.col[[j]]] <- z[,,1]
+            # epistasis - need to reshape the matrix
+            results[wh.col[[j]],wh.col[[i]]] <- matrix(z[,,2],n.pos[j],n.pos[i])
+          }
         }
       }
       else if(model=="normal" && (method=="hk" || method=="em")) {
@@ -347,7 +403,13 @@ function(cross, chr, pheno.col=1,
               resid0 <- lm(pheno ~ ac, weights=weights^2)$resid
             else
               resid0 <- lm(pheno ~ 1, weights=weights^2)$resid
-            if(method=="hk") nllik0 <- (n.ind/2)*log10(sum((resid0*weights)^2))
+            if(method=="hk") {
+              if(n.phe == 1)
+                nllik0 <- (n.ind/2)*log10(sum((resid0*weights)^2))
+              else # multiple phenotypes
+                nllik0 <- apply(resid0, 2, function(x)
+                                (n.ind/2)*log10(sum((x*weights)^2)))
+            }
             else {
               sig0 <- sqrt(sum((resid0*weights)^2)/n.ind)
               nllik0 <- -sum(dnorm(resid0,0,sig0/weights,log=TRUE))/log(10)
@@ -357,7 +419,7 @@ function(cross, chr, pheno.col=1,
 
           if(verbose>1) cat("  --Calculating joint probs.\n")
 
-          if(chrtype[i]=="X" && (type=="bc" || type=="f2" || type=="f2ss")) {
+          if(chrtype[i]=="X" && (type=="bc" || type=="f2")) {
             # calculate joint genotype probabilities for all pairs of positions
             stp <- attr(oldXchr$geno[[1]]$prob, "step")
             oe <- attr(oldXchr$geno[[1]]$prob, "off.end")
@@ -391,14 +453,12 @@ function(cross, chr, pheno.col=1,
           }
 
           # revise pair probilities for X chromosome
-          if(chrtype[i]=="X" && (type=="bc" || type=="f2" || type=="f2ss")) {
+          if(chrtype[i]=="X" && (type=="bc" || type=="f2")) 
             temp <- reviseXdata(type, "full", sexpgm, pairprob=temp)
-            temp[temp < 1e-5] <- 1e-5 # << temp fix for problems with X chromosome
-          }
 
           if(verbose>1) cat("  --Done.\n")
 
-          if(method=="hk") 
+          if(method=="hk") {
             z <- .C("R_scantwo_1chr_hk", 
                     as.integer(n.ind),
                     as.integer(n.pos[i]),
@@ -410,10 +470,21 @@ function(cross, chr, pheno.col=1,
                     as.double(ic),
                     as.integer(n.ic),
                     as.double(pheno),
+                    as.integer(n.phe),
                     as.double(weights),
-                    result=as.double(rep(0,n.pos[i]^2)),
+                    result=as.double(rep(0,n.pos[i]^2*n.phe)),
+                    as.integer(n.col2drop),
+                    as.integer(col2drop),
                     PACKAGE="qtl")
-          else
+            ## fill results matrix
+            if(n.phe == 1) 
+              results[wh.col[[i]],wh.col[[i]]] <-
+                matrix(z$result,ncol=n.pos[i])
+            else # multiple phenotypes
+              results[wh.col[[i]],wh.col[[i]],] <-
+                array(z$result,c(n.pos[i],n.pos[i], n.phe))
+          }
+          else {
             z <- .C("R_scantwo_1chr_em", 
                     as.integer(n.ind),
                     as.integer(n.pos[i]),
@@ -429,16 +500,18 @@ function(cross, chr, pheno.col=1,
                     as.integer(maxit),
                     as.double(tol),
                     as.integer(verbose),
+                    as.integer(n.col2drop),
+                    as.integer(col2drop),
                     PACKAGE="qtl")
-
-          rm(temp) # remove the joint genotype probabilities
-
           # re-organize results
-          results[wh.col[[i]],wh.col[[i]]] <-
-            matrix(z$result,ncol=n.pos[i])
+            results[wh.col[[i]],wh.col[[i]]] <-
+              matrix(z$result,ncol=n.pos[i])
+          }
+          rm(temp) # remove the joint genotype probabilities
         } # end same chromosome
+        
         else {
-          if(method=="hk")
+          if(method=="hk") {
             z <- .C("R_scantwo_2chr_hk",
                     as.integer(n.ind),
                     as.integer(n.pos[i]),
@@ -452,11 +525,26 @@ function(cross, chr, pheno.col=1,
                     as.double(ic),
                     as.integer(n.ic),
                     as.double(pheno),
+                    as.integer(n.phe),
                     as.double(weights),
-                    full=as.double(rep(0,n.pos[i]*n.pos[j])),
-                    int=as.double(rep(0,n.pos[i]*n.pos[j])),
+                    full=as.double(rep(0,n.pos[i]*n.pos[j]*n.phe)),
+                    int=as.double(rep(0,n.pos[i]*n.pos[j]*n.phe)),
                     PACKAGE="qtl")
-          else 
+            ## reorgnize results
+            if(n.phe == 1) {
+              results[wh.col[[j]],wh.col[[i]]] <-
+                matrix(z$full,ncol=n.pos[j])
+              results[wh.col[[i]],wh.col[[j]]] <-
+                matrix(z$int,ncol=n.pos[j])
+            }
+            else { # multiple phenotypes
+              results[wh.col[[j]],wh.col[[i]],] <-
+                array(z$full,c(n.pos[j], n.pos[i], n.phe))
+              results[wh.col[[i]],wh.col[[j]],] <-
+                array(z$int,c(n.pos[j], n.pos[i], n.phe))
+            }
+          }
+          else  {
             z <- .C("R_scantwo_2chr_em",
                     as.integer(n.ind),
                     as.integer(n.pos[i]),
@@ -478,11 +566,12 @@ function(cross, chr, pheno.col=1,
                     as.integer(verbose),
                     PACKAGE="qtl")
 
-          results[wh.col[[j]],wh.col[[i]]] <-
-            t(matrix(z$full,ncol=n.pos[j]))
-          results[wh.col[[i]],wh.col[[j]]] <-
-            matrix(z$int,ncol=n.pos[j])
-        } # end same chromosome
+            results[wh.col[[j]],wh.col[[i]]] <-
+              t(matrix(z$full,ncol=n.pos[j]))
+            results[wh.col[[i]],wh.col[[j]]] <-
+              matrix(z$int,ncol=n.pos[j])
+          }
+        } # end different chromosome
       }
       else if(model=="binary" && method=="em") {
         if(i==j) { # same chromosome
@@ -502,9 +591,12 @@ function(cross, chr, pheno.col=1,
                      nullcoef[-1],rep(0,n.gen[i]*n.gen[i]+
                                       (n.gen[i]*(n.gen[i]-1)*n.ic)))
 
+          if(n.col2drop)
+            start <- c(start[!col2drop], rep(0,sum(col2drop)))
+
           if(verbose>1) cat("  --Calculating joint probs.\n")
 
-          if(chrtype[i]=="X" && (type=="bc" || type=="f2" || type=="f2ss")) {
+          if(chrtype[i]=="X" && (type=="bc" || type=="f2")) {
             # calculate joint genotype probabilities for all pairs of positions
             stp <- attr(oldXchr$geno[[1]]$prob, "step")
             oe <- attr(oldXchr$geno[[1]]$prob, "off.end")
@@ -538,10 +630,8 @@ function(cross, chr, pheno.col=1,
           }
 
           # revise pair probilities for X chromosome
-          if(chrtype[i]=="X" && (type=="bc" || type=="f2" || type=="f2ss")) {
+          if(chrtype[i]=="X" && (type=="bc" || type=="f2")) 
             temp <- reviseXdata(type, "full", sexpgm, pairprob=temp)
-            temp[temp < 1e-5] <- 1e-5 # << temp fix for problems with X chromosome
-          }
 
           if(verbose>1) cat("  --Done.\n")
 
@@ -560,6 +650,8 @@ function(cross, chr, pheno.col=1,
                   as.integer(maxit),
                   as.double(tol),
                   as.integer(verbose),
+                  as.integer(n.col2drop),
+                  as.integer(col2drop),
                   PACKAGE="qtl")
 
           rm(temp) # remove the joint genotype probabilities
@@ -604,9 +696,13 @@ function(cross, chr, pheno.col=1,
         # replace missing and partially informative genotypes with 0's
         datai <- cross$geno[[i]]$data
         datai[is.na(datai)] <- 0
-        if(type=="f2" || type=="f2ss") datai[datai>3] <- 0
+
+        if(type=="f2") datai[datai>3] <- 0
         else if(type=="4way") datai[datai>4] <- 0
 
+        if(chrtype[i]=="X" && (type=="bc" || type=="f2"))
+            datai <- reviseXdata(type, "full", sexpgm, geno=datai)
+
         if(i==j) { # same chromosome
 
           z <- .C("R_scantwo_1chr_mr",
@@ -621,6 +717,8 @@ function(cross, chr, pheno.col=1,
                   as.double(pheno),
                   as.double(weights),
                   result=as.double(rep(0,n.pos[i]^2)),
+                  as.integer(n.col2drop),
+                  as.integer(col2drop),
                   PACKAGE="qtl")
 
           # re-organize results
@@ -632,9 +730,12 @@ function(cross, chr, pheno.col=1,
           # replace missing and partially informative genotypes with 0's
           dataj <- cross$geno[[j]]$data
           dataj[is.na(dataj)] <- 0
-          if(type=="f2" || type=="f2ss") dataj[dataj>3] <- 0
+          if(type=="f2") dataj[dataj>3] <- 0
           else if(type=="4way") dataj[dataj>4] <- 0
 
+          if(chrtype[j]=="X" && (type=="bc" || type=="f2"))
+            dataj <- reviseXdata(type, "full", sexpgm, geno=dataj)
+
           z <- .C("R_scantwo_2chr_mr",
                   as.integer(n.ind),
                   as.integer(n.pos[i]),
@@ -663,8 +764,21 @@ function(cross, chr, pheno.col=1,
     } # end loop over second chr
   } # end loop over first chromosome
 
-  if(method=="hk" || method=="em") # subtr null neg log lik from lower tri
+  # subtract null neg log lik from lower tri
+  if(method=="em")
     results[lower.tri(results)] <- nllik0 - results[lower.tri(results)]
+  else if(method=="hk") {
+    if(n.phe == 1)
+      results[lower.tri(results)] <- nllik0 - results[lower.tri(results)]
+    else { # multiple phenotypes
+      lower <- lower.tri(results[,,1])
+      for(itmp in 1:n.phe) {
+        # I'm doing a loop here. I should put null model back to C function
+        results[,,itmp][lower] <- nllik0[itmp] -
+          results[,,itmp][lower]
+      }
+    }
+  }
 
   # If the X chromosome was included, need to do an adjustment...
   scanoneX <- NULL
@@ -677,8 +791,19 @@ function(cross, chr, pheno.col=1,
     sexpgmcovar <- temp$sexpgmcovar
       
     if(adjustX) {
-      if(method=="mr") {
-        warning("The X chr may not be working properly for scantwo with method mr.") 
+      if(method=="mr" && any(is.na(pull.geno(cross)))) 
+        warning("Scantwo with the X chr doesn't work quite right when method=\"mr\"\n",
+                "    when there is missing genotype data.")
+
+      if(model=="binary") {
+        if(n.ac > 0)
+          nullfitX <- glm(pheno ~ ac+sexpgmcovar, family=binomial(link="logit"))
+        else
+          nullfitX <- glm(pheno ~ sexpgmcovar, family=binomial(link="logit"))
+        fittedX <- nullfitX$fitted
+        nullcoefX <- nullfitX$coef
+        nllikX <- -sum(pheno*log10(fittedX) + (1-pheno)*log10(1-fittedX))
+        if(verbose > 1) cat("X chr null log lik: ", nllikX, "\n")
       }
       else {
         if(n.ac > 0) {
@@ -690,9 +815,15 @@ function(cross, chr, pheno.col=1,
         else 
           residX <- lm(pheno ~ sexpgmcovar, weights=weights^2)$resid
 
-        if(method=="hk") nllikX <- (n.ind/2)*log10(sum((residX*weights)^2))
+        if(method=="hk") {
+          if(n.phe==1) 
+            nllikX <- (n.ind/2)*log10(sum((residX*weights)^2))
+          else 
+            nllikX <- (n.ind/2)*apply(residX, 2, function(a,b)
+                                    log10(sum((a*b)^2)), weights)
+        }
         else {
-          if(method=="imp") {
+          if(method=="imp" || method=="mr") {
             if(n.ac > 0) {
               out0 <- lm(pheno ~ ac, weights=weights^2)
               resid0 <- out0$resid
@@ -701,55 +832,95 @@ function(cross, chr, pheno.col=1,
               out0 <- lm(pheno ~ 1, weights=weights^2)
               resid0 <- out0$resid
             }
-            
-            sig0 <- sqrt(sum((resid0*weights)^2)/n.ind)
-            nllik0 <- -sum(dnorm(resid0,0,sig0/weights,log=TRUE))/log(10)
+          
+            if(n.phe > 1) {
+              sig0 <- sqrt(apply(resid0, 2, function(a,b) sum((a*b)^2),weights)/n.ind)
+              nllik0 <- sig0
+              for(i in seq(along=nllik0))
+                nllik0[i] <- -sum(dnorm(resid0[,i],0,sig0[i]/weights,log=TRUE))/log(10)
+            }
+            else {
+              sig0 <- sqrt(sum((resid0*weights)^2)/n.ind)
+              nllik0 <- -sum(dnorm(resid0,0,sig0/weights,log=TRUE))/log(10)
+            }
           }
-          sigX <- sqrt(sum((residX*weights)^2)/n.ind)
-          nllikX <- -sum(dnorm(residX,0,sigX/weights,log=TRUE))/log(10)
-        }
-
-        wh <- ((gmap[row(results),4] | gmap[col(results),4]) & lower.tri(results))
-        results[wh] <- results[wh] + nllikX - nllik0
-
-        if(run.scanone) {
-          if(verbose) cat(" --Running scanone with special X chr covariates\n")
-
-          notxchr <- which(sapply(cross$geno,class)!="X")
-          if(length(notxchr) > 0) {
-            temp <- scanone(subset(cross,chr=notxchr),
-                            pheno.col=pheno.col, method=method,
-                            addcovar=cbind(ac,sexpgmcovar),
-                            intcovar=ic, weights=weights,
-                            maxit=maxit, tol=tol, verbose=FALSE)
-
-
-            nam <- rownames(temp)
-            scanoneX <- temp[,3]
-            names(scanoneX) <- nam
-
-            scanoneX <- c(scanoneX,out.scanone[rownames(gmap)][gmap[,4]])
-            scanoneX <- scanoneX[rownames(gmap)]
+ 
+          if(n.phe > 1) {
+            sigX <- sqrt(apply(residX, 2, function(a,b) sum((a*b)^2),weights)/n.ind)
+            nllikX <- sigX
+            for(i in seq(along=nllikX))
+              nllikX[i] <- -sum(dnorm(residX[,i],0,sigX[i]/weights,log=TRUE))/log(10)
           }
           else {
-            scanoneX <- out.scanone[rownames(gmap)][gmap[,4]]
-            scanoneX <- scanoneX[rownames(gmap)]
+            sigX <- sqrt(sum((residX*weights)^2)/n.ind)
+            nllikX <- -sum(dnorm(residX,0,sigX/weights,log=TRUE))/log(10)
           }
         }
       }
+
+      if(n.phe > 1)
+        wh <- ((gmap[row(results[,,1]),4] | gmap[col(results[,,1]),4]) &
+               lower.tri(results[,,1]))
+      else 
+        wh <- ((gmap[row(results),4] | gmap[col(results),4]) & lower.tri(results))
+
+      if(n.phe > 1) {
+        for(i in 1:n.phe) 
+          results[,,i][wh] <- results[,,i][wh] + nllikX[i] - nllik0[i]
+      }
+      else 
+        results[wh] <- results[wh] + nllikX - nllik0
+
+      if(run.scanone) {
+        notxchr <- which(sapply(cross$geno,class)!="X")
+        if(length(notxchr) > 0) {
+          if(verbose) cat(" --Running scanone with special X chr covariates\n")
+          temp <- scanone(subset(cross,chr=notxchr),
+                          pheno.col=pheno.col, 
+                          model=model, method=method,
+                          addcovar=cbind(ac,sexpgmcovar),
+                          intcovar=ic, weights=weights,
+                          maxit=maxit, tol=tol, verbose=FALSE)
+
+          scanoneX <- temp[,-(1:2),drop=FALSE]
+
+          scanoneX <- rbind(scanoneX,
+                            out.scanone[rownames(gmap),,drop=FALSE][gmap[,4],,drop=FALSE])
+
+          scanoneX <- scanoneX[rownames(gmap),,drop=FALSE]
+        }
+        else {
+          scanoneX <- out.scanone[rownames(gmap),,drop=FALSE][gmap[,4],,drop=FALSE]
+          scanoneX <- scanoneX[rownames(gmap),,drop=FALSE]
+        }
+      }
     }
   }
   
-
-  if(any(is.na(results) | results < -1e-6 | results == Inf))
-    warning("Some LOD scores NA, Inf or < 0")
+#  if(any(is.na(results) | results < -1e-6 | results == Inf))
+#    warning("Some LOD scores NA, Inf or < 0")
+  if(any(is.na(results)))
+    warning("Some LOD scores NA")
+  if(any(!is.na(results) & results < 0))
+    warning("Some LOD scores < 0")
+  if(any(!is.na(results) & (results == Inf | results == -Inf)))
+    warning("Some LOD scores = Inf or -Inf")
   
+  if(!is.null(scanoneX)) scanoneX <- as.matrix(scanoneX)
+
   # output has 2 fields, lod and map
   out <- list(lod=results,map=gmap,scanoneX=scanoneX)
   class(out) <- "scantwo"
 
-  if(run.scanone) # also did scanone
-    diag(out$lod) <- out.scanone[rownames(out$map)]
+  # fill in scanone result
+  if(run.scanone) { 
+    if(n.phe == 1)
+      diag(out$lod) <- out.scanone[rownames(out$map),]
+    else {
+      for(iphe in 1:n.phe) 
+        diag(out$lod[,,iphe]) <- out.scanone[rownames(out$map),iphe]
+    }
+  }
 
   attr(out,"method") <- method
   attr(out,"type") <- type
@@ -763,67 +934,177 @@ function(cross, chr, pheno.col=1,
 ######################################################################
 
 scantwo.perm <-
-function(cross, pheno.col=1,
+function(cross, pheno.col=1, model=c("normal","binary"),
          method=c("em","imp","hk","mr","mr-imp","mr-argmax"),
-         model=c("normal","binary"),
          addcovar=NULL, intcovar=NULL, weights=NULL,
          incl.markers=FALSE, maxit=4000, tol=1e-4, verbose=FALSE,
-         n.perm=1000) 
+         n.perm=1000, n.node=1) 
 {
   method <- match.arg(method)
   model <- match.arg(model)
 
-  n.ind <- nind(cross)
-  addcovarp <- intcovarp <- NULL
-  if(!is.null(addcovar)) addcovar <- as.matrix(addcovar)
-  if(!is.null(intcovar)) intcovar <- as.matrix(intcovar)
-
-  if(method=="mr-imp") # save version with missing genotypes 
-    tempcross <- cross
-  if(method=="mr-argmax") # impute genotypes
-    cross <- fill.geno(cross,method="argmax")
-
-  # initialize the result matrix
-  # the first row is for full model comparison
-  # the second row is for additive model comparison
-  res <- matrix(ncol=2,nrow=n.perm)
-  for(i in 1:n.perm) {
-    if(verbose) cat("Permutation", i, "\n")
-
-    # impute genotypes for method "mr-imp"
-    if(method=="mr-imp") cross <- fill.geno(tempcross)
-
-    o <- sample(1:n.ind)
-    cross$pheno <- cross$pheno[o,,drop=FALSE]
-    if(!is.null(addcovar)) addcovarp <- addcovar[o,,drop=FALSE]
-    if(!is.null(intcovar)) intcovarp <- intcovar[o,,drop=FALSE]
-    tem <- scantwo(cross,  pheno.col=pheno.col,
-                   method=method, model=model, addcovar=addcovarp,
-                   intcovar=intcovarp, incl.markers=incl.markers,
-                   weights=weights, run.scanone=FALSE, maxit=maxit,
-                   tol=tol, verbose=FALSE, n.perm = -1)
-
-    # take max of the two triangles
-    res[i,1] <- max( tem$lod[tem$lod < Inf & row(tem$lod)>col(tem$lod)], na.rm=TRUE )
-    res[i,2] <- max( tem$lod[tem$lod < Inf & row(tem$lod)<col(tem$lod)], na.rm=TRUE )
+  # for running on clusters, check if the setup is correct
+  if(n.node > 1) { ## run on cluster
+    ## initialize cluster
+    if(!require(snow))
+      stop("You have to install SNOW package to use cluster")
+    ## make cluster object
+    cl <- makeMPIcluster(nnodes)
+    ## correct the possible correlation problem
+    clusterApply(cl, runif(length(cl), max=1000000000), set.seed)
+    ## turn off verbose
+    verbose <- FALSE
   }
-  colnames(res) <- c("LOD.jnt","LOD.interxn")
-  attr(res,"method") <- method
+  
+  ## start permutation
+  if(n.node > 1) { ## run on clusters
+    ## calculate the number of permutation needed in each node
+    nperm.cluster <- rep(floor((n.perm-1)/n.node), n.node)
+    ## maybe some leftovers
+    leftover <- n.perm - 1 - sum(nperm.cluster)
+    if(leftover > 0)
+      nperm.cluster[1:leftover] <- nperm.cluster[1:leftover] + 1
+    ## load library on all nodes
+    clusterEvalQ(cl, library(qtl))
+    ## use clusterApply to run permutation on all nodes
+    ## note that the only different parameter passed to function
+    ## is nperm.cluster.
+    cat(paste("Doing permutation on", nnodes, "cluster nodes ... \n"))
+    pstar.nodes <- clusterApply(cl, nperm.cluster, scantwo.perm.engine,
+                                cross, pheno.col, model, method,
+                                addcovar, intcovar, weights,
+                                incl.markers, maxit, tol, verbose)
+    
+    ## collect the result from clusters
+    ## this part is to be done !!! - hao wu
+  }
+  else { ## on single node
+    res <- scantwo.perm.engine(n.perm, cross, pheno.col, model, 
+                               method, addcovar, intcovar, weights,
+                               incl.markers, maxit, tol, verbose)
+  }
+  
   res
 }
 
 
+######################################################################
+#
+# Engine function to scantwo permutation
+#
+######################################################################
+scantwo.perm.engine <-
+function(n.perm, cross, pheno.col, model,
+         method, addcovar, intcovar, weights,
+         incl.markers, maxit, tol, verbose)
+{
+  ## local variables
+  n.phe <- length(pheno.col)
+#  n.addcov <- ncol(addcovar)
+  n.ind <- dim(cross$pheno)[1]
+
+  ## initialize result
+  ## it's a list of two components, LOD.jnt and LOD.interxn
+  LOD.jnt <- matrix(0, n.perm, n.phe)
+  LOD.interxn <- matrix(0, n.perm, n.phe)
+#  res <- matrix(ncol=2,nrow=n.perm)
+      
+  if( (n.phe == 1) && ((method=="imp") || (method=="hk")) &&
+       is.null(addcovar) && is.null(intcovar) ) {
+    if(verbose)
+      cat("Doing permutation in batch mode ...\n")
+    ## if there's only one phenotype, no covariate, and method is imp or hk,
+    ## generate permuted phenotype as a matrix and do permutation
+    ## as multiple phenotypes
+    ord <- matrix(0, n.ind, n.perm)
+    for(iperm in 1:n.perm)
+      ord[,iperm] <- sample(n.ind)
+
+    cross$pheno <- matrix(cross$pheno[,pheno.col][ord], nrow=n.ind)
+
+    pheno.col <- 1:n.perm
+    tem <- scantwo(cross,,pheno.col,model,method,addcovar,
+                   intcovar,weights,run.scanone=FALSE,
+                   maxit=maxit,incl.markers=incl.markers,
+                   tol=tol, verbose=TRUE, n.perm=-1)
+    ## find the maximum LOD on each permutation
+    for(iperm in 1:n.perm) {
+      tmplod <- tem$lod[,,iperm]
+      LOD.jnt[iperm, 1] <- max(tmplod[tmplod < Inf & row(tmplod)>col(tmplod)],
+                               na.rm=TRUE )
+      LOD.interxn[iperm,1] <- max(tmplod[tmplod < Inf & row(tmplod)<col(tmplod)],
+                                  na.rm=TRUE )
+    }
+  }
+  else { ## all other cases, do one permutation at a time
+    if(method=="mr-imp") # save version with missing genotypes
+      tempcross <- cross
+    if(method=="mr-argmax") # impute genotypes
+      cross <- fill.geno(cross,method="argmax")
+    addcovarp <- intcovarp <- NULL
+    if(!is.null(addcovar)) addcovar <- as.matrix(addcovar)
+    if(!is.null(intcovar)) intcovar <- as.matrix(intcovar)
+    ## permutation loop
+    for(i in 1:n.perm) {
+      if(verbose) cat("Permutation", i, "\n")
+      ## impute genotypes for method "mr-imp"
+      if(method=="mr-imp") cross <- fill.geno(tempcross)
+
+      o <- sample(1:n.ind)
+      cross$pheno <- cross$pheno[o,,drop=FALSE]
+      if(!is.null(addcovar)) addcovarp <- addcovar[o,,drop=FALSE]
+      if(!is.null(intcovar)) intcovarp <- intcovar[o,,drop=FALSE]
+      tem <- scantwo(cross,  pheno.col=pheno.col, model=model, 
+                     method=method, addcovar=addcovarp,
+                     intcovar=intcovarp, incl.markers=incl.markers,
+                     weights=weights, run.scanone=FALSE, maxit=maxit,
+                     tol=tol, verbose=FALSE, n.perm = -1)
+
+      ## calculate max joint lod and interactive lod for each phenotype
+      ## it's to take max of the two triangles for each trait
+      for(iphe in 1:n.phe) {
+        if(n.phe == 1)
+          tmplod <- tem$lod
+        else
+          tmplod <- tem$lod[,,iphe]
+        LOD.jnt[i, iphe] <- max(tmplod[tmplod < Inf & row(tmplod)>col(tmplod)],
+                                na.rm=TRUE )
+        LOD.interxn[i,iphe] <- max(tmplod[tmplod < Inf & row(tmplod)<col(tmplod)],
+                                   na.rm=TRUE )
+      }
+    }
+
+  }
+
+  ## make result
+  res <- list(LOD.jnt=LOD.jnt, LOD.interxn=LOD.interxn)
+  attr(res,"method") <- method
+  # return
+  res
+}
+
 ######################################################################
 #
 # summerize the result from scantwo
 #
 ######################################################################
 summary.scantwo <-
-function (object, thresholds = c(0, 0, 0),
+function (object, thresholds = c(0, 0, 0), lodcolumn=1,
           type = c("joint","interaction"), ...)
 {
   type <- match.arg(type)
   
+  if(length(dim(object$lod)) > 2) { # results from multiple phenotypes
+    if(length(lodcolumn) > 1) {
+      warning("Argument lodcolumn should be of length 1.")
+      lodcolumn <- lodcolumn[1]
+    }
+      
+    if(lodcolumn < 0 || lodcolumn > dim(object$lod)[3])
+      stop("Argument lodcolumn misspecified.")
+    object$lod <- object$lod[,,lodcolumn]
+  }
+
   if(length(thresholds) < 3) {
     if(length(thresholds) == 1) thresholds <- c(thresholds, 0, 0)
     else stop("You must give three thresholds: full, interaction and main\n")
