@@ -2,9 +2,9 @@
  * 
  * scantwo_binary_em.c
  *
- * copyright (c) 2004-5, Karl W Broman, Johns Hopkins University
+ * copyright (c) 2004-6, Karl W Broman, Johns Hopkins University
  *
- * last modified Apr, 2005
+ * last modified Feb, 2006
  * first written Dec, 2004
  *
  * Licensed under the GNU General Public License version 2 (June, 1991)
@@ -12,7 +12,8 @@
  * C functions for the R/qtl package
  *
  * These functions are for performing a 2-dimensional genome scan 
- * with a 2-QTL model by interval mapping.(the EM algorithm).
+ * with a 2-QTL model by interval mapping (the EM algorithm) for  
+ * a binary trait.
  *
  * Contains: R_scantwo_1chr_binary_em, scantwo_1chr_binary_em, 
  *           R_scantwo_2chr_binary_em, scantwo_2chr_binary_em,
@@ -27,6 +28,7 @@
 #include <Rmath.h>
 #include <R_ext/PrtUtil.h>
 #include <R_ext/Applic.h>
+#include <R_ext/Linpack.h>
 #include "util.h"
 #include "scantwo_binary_em.h"
 #define TOL 1e-12
@@ -44,8 +46,9 @@
 void R_scantwo_1chr_binary_em(int *n_ind, int *n_pos, int *n_gen,
 			      double *pairprob, double *addcov, int *n_addcov, 
 			      double *intcov, int *n_intcov, int *pheno, 
-			      double *start,
-			      double *result, int *maxit, double *tol, int *verbose)
+			      double *start, double *result, int *maxit, 
+			      double *tol, int *verbose, int *n_col2drop,
+			      int *col2drop)
 {
   double **Result, **Addcov, **Intcov, *****Pairprob;
 
@@ -58,7 +61,8 @@ void R_scantwo_1chr_binary_em(int *n_ind, int *n_pos, int *n_gen,
 
   scantwo_1chr_binary_em(*n_ind, *n_pos, *n_gen, Pairprob, 
 			 Addcov, *n_addcov, Intcov, *n_intcov, 
-			 pheno, start, Result, *maxit, *tol, *verbose);
+			 pheno, start, Result, *maxit, *tol, *verbose,
+			 *n_col2drop, col2drop);
 }
 
 /**********************************************************************
@@ -110,15 +114,38 @@ void R_scantwo_1chr_binary_em(int *n_ind, int *n_pos, int *n_gen,
 void scantwo_1chr_binary_em(int n_ind, int n_pos, int n_gen, 
 			    double *****Pairprob, double **Addcov, int n_addcov, 
 			    double **Intcov, int n_intcov, int *pheno, double *start,
-			    double **Result, int maxit, double tol, int verbose)
+			    double **Result, int maxit, double tol, int verbose,
+			    int n_col2drop, int *col2drop)
 {
-  int error_flag, i1, i2, k1, k2, j, m, n_col[2], nit[2], r, flag=0;
+  int error_flag, i, i1, i2, k1, k2, j, m, n_col[2], n_col_rev[2], nit[2], r, flag=0;
   double *param, *oldparam, ***Wts12, pardif;
   double *wts, ***Probs, oldllik=0.0, llik[2];
+  int *allcol2drop;
 
   n_col[0] = (2*n_gen-1) + n_addcov + 2*(n_gen-1)*n_intcov;
   n_col[1] = n_gen*n_gen + n_addcov + (n_gen*n_gen-1)*n_intcov;
 
+  /* expand col2drop */
+  if(n_col2drop) {
+    allocate_int(n_col[1], &allcol2drop);
+    expand_col2drop(n_gen, n_addcov, n_intcov, 
+		    col2drop, allcol2drop);
+  }
+
+  /* revised numbers of parameters */
+  if(n_col2drop) {
+    n_col_rev[0] = 0;
+    for(i=0; i<n_col[0]; i++) 
+      if(!allcol2drop[i]) n_col_rev[0]++;
+    n_col_rev[1] = n_col_rev[0];
+    for(i=n_col[0]; i<n_col[1]; i++)
+      if(!allcol2drop[i]) n_col_rev[1]++;
+  }
+  else {
+    n_col_rev[0] = n_col[0];
+    n_col_rev[1] = n_col[1];
+  }
+
   /* allocate workspaces */
   wts = (double *)R_alloc(2*n_gen*(n_gen+1)*n_ind, sizeof(double));
   reorg_genoprob(n_ind, n_gen, n_gen, wts+2*n_gen*n_ind, &Wts12);
@@ -144,7 +171,8 @@ void scantwo_1chr_binary_em(int n_ind, int n_pos, int n_gen,
 
 	scantwo_binary_em_mstep(n_ind, n_gen, n_gen, Addcov, n_addcov, 
 				Intcov, n_intcov, pheno, Probs, 
-				oldparam, m, n_col[m], &error_flag);
+				oldparam, m, n_col[m], &error_flag,
+				n_col2drop, allcol2drop, verbose);
 	if(error_flag) {
 	  if(verbose>1) 
 	    Rprintf("   [%3d %3d] %1d: Initial model had error.\n",
@@ -153,7 +181,7 @@ void scantwo_1chr_binary_em(int n_ind, int n_pos, int n_gen,
 	else { /* only proceed if there's no error */
 	  oldllik = scantwo_binary_em_loglik(n_ind, n_gen, n_gen, Probs, Addcov, 
 					     n_addcov, Intcov, n_intcov, pheno, 
-					     oldparam, m);
+					     oldparam, m, n_col2drop, allcol2drop);
 	  if(verbose>2) 
 	    Rprintf("   [%3d %3d] %1d %9.3lf\n", 
 		    i1+1, i2+1, m+1, oldllik);
@@ -163,11 +191,13 @@ void scantwo_1chr_binary_em(int n_ind, int n_pos, int n_gen,
 	  for(r=0; r<maxit; r++) { /* loop over iterations */
 	    scantwo_binary_em_estep(n_ind, n_gen, n_gen, Probs, Wts12, 
 				    Addcov, n_addcov, Intcov, 
-				    n_intcov, pheno, oldparam, m, 1);
+				    n_intcov, pheno, oldparam, m, 1,
+				    n_col2drop, allcol2drop);
 
 	    scantwo_binary_em_mstep(n_ind, n_gen, n_gen, Addcov, n_addcov, 
 				    Intcov, n_intcov, pheno, Wts12, 
-				    param, m, n_col[m], &error_flag);
+				    param, m, n_col[m], &error_flag,
+				    n_col2drop, allcol2drop, verbose);
 	    if(error_flag) {
 	      flag=0;
 	      if(verbose>1)
@@ -178,7 +208,7 @@ void scantwo_1chr_binary_em(int n_ind, int n_pos, int n_gen,
 
 	    llik[m] = scantwo_binary_em_loglik(n_ind, n_gen, n_gen, Probs, Addcov, 
 					       n_addcov, Intcov, n_intcov, pheno, 
-					       param, m);
+					       param, m, n_col2drop, allcol2drop);
 
 	    if(verbose>1) {
 	      if(verbose>2) {
@@ -195,7 +225,7 @@ void scantwo_1chr_binary_em(int n_ind, int n_pos, int n_gen,
 			i1+1, i2+1, m+1, r+1, (llik[m]-oldllik));
 
 	      if(verbose>3) { /* print parameters */
-		for(j=0; j<n_col[m]; j++) 
+		for(j=0; j<n_col_rev[m]; j++) 
 		  Rprintf(" %7.3lf", param[j]);
 		Rprintf("\n");
 	      }
@@ -203,7 +233,7 @@ void scantwo_1chr_binary_em(int n_ind, int n_pos, int n_gen,
 	    
 	    flag = 1;
 	    /* use log likelihood only to check for convergence */
-	    if(llik[m]-oldllik < tol) { 
+	    if((llik[m]-oldllik) < tol) { 
 	      flag = 0; 
 	      break; 
 	    }
@@ -338,6 +368,7 @@ void scantwo_2chr_binary_em(int n_ind, int n_pos1, int n_pos2, int n_gen1,
   int error_flag, i1, i2, k1, k2, j, m, n_col[2], nit[2], r, flag=0;
   double *param, *oldparam, ***Wts12;
   double *wts, ***Probs, oldllik=0.0, llik[2];
+  int n_col2drop=0, *allcol2drop=0;
 
   n_col[0] = (n_gen1+n_gen2-1) + n_addcov + (n_gen1+n_gen2-2)*n_intcov;
   n_col[1] = n_gen1*n_gen2 + n_addcov + (n_gen1*n_gen2-1)*n_intcov;
@@ -368,7 +399,8 @@ void scantwo_2chr_binary_em(int n_ind, int n_pos1, int n_pos2, int n_gen1,
 
 	scantwo_binary_em_mstep(n_ind, n_gen1, n_gen2, Addcov, n_addcov, 
 				Intcov, n_intcov, pheno, Probs, 
-				oldparam, m, n_col[m], &error_flag);
+				oldparam, m, n_col[m], &error_flag,
+				n_col2drop, allcol2drop, verbose);
 
 	if(error_flag) {
 	  if(verbose>1)
@@ -378,7 +410,9 @@ void scantwo_2chr_binary_em(int n_ind, int n_pos1, int n_pos2, int n_gen1,
 	else { /* only proceed if there's no error */
 	  oldllik = scantwo_binary_em_loglik(n_ind, n_gen1, n_gen2, Probs, 
 					     Addcov, n_addcov, Intcov, 
-					     n_intcov, pheno, oldparam, m);
+					     n_intcov, pheno, oldparam, m,
+					     n_col2drop, allcol2drop);
+
 	  if(verbose>2)
 	    Rprintf("   [%3d %3d] %1d %9.3lf\n", 
 		    i1+1, i2+1, m+1, oldllik);
@@ -387,12 +421,15 @@ void scantwo_2chr_binary_em(int n_ind, int n_pos1, int n_pos2, int n_gen1,
 
 	  for(r=0; r<maxit; r++) { /* loop over iterations */
 	    scantwo_binary_em_estep(n_ind, n_gen1, n_gen2, Probs, Wts12, 
-			     Addcov, n_addcov, Intcov, 
-			     n_intcov, pheno, oldparam, m, 1);
+				    Addcov, n_addcov, Intcov, 
+				    n_intcov, pheno, oldparam, m, 1,
+				    n_col2drop, allcol2drop);
 
 	    scantwo_binary_em_mstep(n_ind, n_gen1, n_gen2, Addcov, n_addcov, 
 				    Intcov, n_intcov, pheno, Wts12, 
-				    param, m, n_col[m], &error_flag);
+				    param, m, n_col[m], &error_flag,
+				    n_col2drop, allcol2drop, verbose);
+
 	    if(error_flag) {
 	      flag=0;
 	      if(verbose>1)
@@ -403,7 +440,8 @@ void scantwo_2chr_binary_em(int n_ind, int n_pos1, int n_pos2, int n_gen1,
 
 	    llik[m] = scantwo_binary_em_loglik(n_ind, n_gen1, n_gen2, Probs, 
 					       Addcov, n_addcov, Intcov, 
-					       n_intcov, pheno, param, m);
+					       n_intcov, pheno, param, m,
+					       n_col2drop, allcol2drop);
 	    
 	    if(verbose>1) { /* print log likelihood */
 	      if(verbose>2)
@@ -422,7 +460,7 @@ void scantwo_2chr_binary_em(int n_ind, int n_pos1, int n_pos2, int n_gen1,
 	    
 	    flag = 1;
 	    /* use log likelihood only to check for convergence */
-	    if(llik[m]-oldllik < tol) { 
+	    if((llik[m]-oldllik) < tol) { 
 	      flag = 0; 
 	      break; 
 	    }
@@ -487,6 +525,7 @@ void scantwo_2chr_binary_em(int n_ind, int n_pos1, int n_pos2, int n_gen1,
  *
  * error_flag     Set to 1 if X'X is singular
  *
+
  **********************************************************************/
 
 void scantwo_binary_em_mstep(int n_ind, int n_gen1, int n_gen2, 
@@ -494,9 +533,11 @@ void scantwo_binary_em_mstep(int n_ind, int n_gen1, int n_gen2,
 			     double **Intcov, int n_intcov, int *pheno, 
 			     double ***Wts12, 
 			     double *param, int full_model,
-			     int n_col, int *error_flag)
+			     int n_col, int *error_flag,
+			     int n_col2drop, int *allcol2drop,
+			     int verbose)
 {
-  int i, j, j2, k1, k2, s, s2, info;
+  int i, j, j2, k1, k2, s, s2, info, ss;
   double rcond, temp, *junk;
   double *grad, *jac, **Jac;
   double *tf1, ***f1, *tf2, ***f2;
@@ -525,54 +566,73 @@ void scantwo_binary_em_mstep(int n_ind, int n_gen1, int n_gen2,
 
   /* calculate fitted values */
   for(i=0; i<n_ind; i++) {
-    s = n_gen1+n_gen2-1;
+    if(n_col2drop) {
+      for(ss=0, s=0; ss<n_gen1+n_gen2-1; ss++) 
+	if(!allcol2drop[ss]) s++;
+    }
+    else s = n_gen1+n_gen2-1;
+
     temp = 0.0;
-    for(j=0; j<n_addcov; j++) 
-      temp += (Addcov[j][i]*param[j+s]);
+    for(j=0; j<n_addcov; j++, s++) 
+      temp += (Addcov[j][i]*param[s]);
   
     /* QTL 1 effect */
-    for(k1=0; k1<n_gen1; k1++) { 
-      for(k2=0; k2<n_gen2; k2++) 
-	fitted[k1][k2] = param[k1]+temp;
+    for(k1=0, ss=0, s=0; k1<n_gen1; k1++, ss++, s++) { 
+      if(!n_col2drop || !allcol2drop[ss]) {
+	for(k2=0; k2<n_gen2; k2++) 
+	  fitted[k1][k2] = param[s]+temp;
+      }
+      else s--;
     }
-    s = n_gen1; /* location in param vector */
 
     /* QTL 2 effect */
-    for(k2=0; k2<n_gen2-1; k2++) { 
-      for(k1=0; k1<n_gen1; k1++) 
-	fitted[k1][k2] += param[k2+s];
+    for(k2=0; k2<n_gen2-1; k2++, ss++, s++) { 
+      if(!n_col2drop || !allcol2drop[ss]) {
+	for(k1=0; k1<n_gen1; k1++) 
+	  fitted[k1][k2] += param[s];
+      }
+      else s--;
     }
-    s += (n_gen2-1+n_addcov);
+    s += n_addcov;
+    ss += n_addcov;
 
     /* QTL x interactive covar */
     for(j=0; j<n_intcov; j++) {
-      for(k1=0; k1<n_gen1-1; k1++) { /* QTL1 x intxn */
-	for(k2=0; k2<n_gen2; k2++)
-	  fitted[k1][k2] += param[k1+s]*Intcov[j][i];
+      for(k1=0; k1<n_gen1-1; k1++, ss++, s++) { /* QTL1 x intxn */
+	if(!n_col2drop || !allcol2drop[ss]) {
+	  for(k2=0; k2<n_gen2; k2++)
+	    fitted[k1][k2] += param[s]*Intcov[j][i];
+	}
+	else s--;
       }
-      s += n_gen1-1;
-      for(k2=0; k2<n_gen2-1; k2++) { /* QTL2 x intxn */
-	for(k1=0; k1<n_gen1; k1++)
-	  fitted[k1][k2] += param[k2+s]*Intcov[j][i];
+
+      for(k2=0; k2<n_gen2-1; k2++, ss++, s++) { /* QTL2 x intxn */
+	if(!n_col2drop || !allcol2drop[ss]) {
+	  for(k1=0; k1<n_gen1; k1++)
+	    fitted[k1][k2] += param[s]*Intcov[j][i];
+	}
+	else s--;
       }
-      s += n_gen2-1;
     }
       
     if(full_model) {
       /* QTL x QTL interaction */
       for(k1=0; k1<n_gen1-1; k1++) 
-	for(k2=0; k2<n_gen2-1; k2++) 
-	  fitted[k1][k2] += param[k1*(n_gen2-1)+k2+s];
-      s += (n_gen1-1)*(n_gen2-1);
-      
+	for(k2=0; k2<n_gen2-1; k2++, ss++, s++) {
+	  if(!n_col2drop || !allcol2drop[ss]) 
+	    fitted[k1][k2] += param[s];
+	  else s--;
+	}
+
       /* QTL x QTL x interactive covar */
       for(j=0; j<n_intcov; j++) {
 	for(k1=0; k1<n_gen1-1; k1++) {
-	  for(k2=0; k2<n_gen2-1; k2++) {
-	    fitted[k1][k2] += param[k1*(n_gen2-1)+k2+s]*Intcov[j][i];
+	  for(k2=0; k2<n_gen2-1; k2++, s++, ss++) {
+	    if(!n_col2drop || !allcol2drop[ss]) 
+	      fitted[k1][k2] += param[s]*Intcov[j][i];
+	    else s--;
 	  }
 	}
-	s += (n_gen1-1)*(n_gen2-1);
       }
     }
 
@@ -588,7 +648,6 @@ void scantwo_binary_em_mstep(int n_ind, int n_gen1, int n_gen2,
     }
   }
     
-
   /* calculate gradient */
   for(i=0; i<n_ind; i++) {
     for(k1=0; k1<n_gen1; k1++)  /* QTL 1 */
@@ -630,8 +689,6 @@ void scantwo_binary_em_mstep(int n_ind, int n_gen1, int n_gen2,
     }
   } /* end loop over individuals */
 
-
-
   /* calculate Jacobian; only the upper right triangle is needed */
   for(i=0; i<n_ind; i++) {
     /* QTL 1 columns */
@@ -771,11 +828,16 @@ void scantwo_binary_em_mstep(int n_ind, int n_gen1, int n_gen2,
   } /* end loop over individuals */
   /* done calculating Jacobian */
 
+  if(n_col2drop) { /* drop some columns (X chromosome) */
+    dropcol_xpy(n_col, allcol2drop, grad);
+
+    dropcol_xpx(&n_col, allcol2drop, jac);
+  }
 
   /* solve work1 * beta = work2 for beta */
   F77_CALL(dpoco)(jac, &n_col, &n_col, &rcond, junk, &info);
   if(fabs(rcond) < TOL || info != 0) { /* error! */
-    warning("X'X matrix is singular.\n");
+    if(verbose > 1) Rprintf("X'X matrix is singular.\n");
     *error_flag = 1;
   }
   else {
@@ -829,65 +891,84 @@ void scantwo_binary_em_estep(int n_ind, int n_gen1, int n_gen2,
 			     double ***Probs, double ***Wts12, 
 			     double **Addcov, int n_addcov, double **Intcov,
 			     int n_intcov, int *pheno, 
-			     double *param, int full_model, int rescale)
+			     double *param, int full_model, int rescale,
+			     int n_col2drop, int *allcol2drop)
 {
-  int i, j, k1, k2, s;
+  int i, j, k1, k2, s, ss;
   double temp;
 
   for(i=0; i<n_ind; i++) {
 
     /* Get fitted values and put in Wts12 */
     /* additive covar effect */
-    s = n_gen1+n_gen2-1;
+    if(n_col2drop) {
+      for(ss=0, s=0; ss<n_gen1+n_gen2-1; ss++)
+	if(!allcol2drop[ss]) s++;
+    }
+    else s=n_gen1+n_gen2-1;
+
     temp = 0.0;
-    for(j=0; j<n_addcov; j++) 
-      temp += (Addcov[j][i]*param[j+s]);
+    for(j=0; j<n_addcov; j++, s++) 
+      temp += (Addcov[j][i]*param[s]);
     
     /* QTL 1 effect */
-    for(k1=0; k1<n_gen1; k1++) { 
-      for(k2=0; k2<n_gen2; k2++) 
-	Wts12[k1][k2][i] = param[k1]+temp;
+    for(k1=0, ss=0, s=0; k1<n_gen1; k1++, ss++, s++) { 
+      if(!n_col2drop || !allcol2drop[ss]) {
+	for(k2=0; k2<n_gen2; k2++) 
+	  Wts12[k1][k2][i] = param[s]+temp;
+      }
+      else s--;
     }
-    s = n_gen1; /* location in param vector */
 
     /* QTL 2 effect */
-    for(k2=0; k2<n_gen2-1; k2++) { 
-      for(k1=0; k1<n_gen1; k1++) 
-	Wts12[k1][k2][i] += param[k2+s];
+    for(k2=0; k2<n_gen2-1; k2++, ss++, s++) { 
+      if(!n_col2drop || !allcol2drop[ss]) {
+	for(k1=0; k1<n_gen1; k1++) 
+	  Wts12[k1][k2][i] += param[s];
+      }
+      else s--;
     }
-    s += (n_gen2-1+n_addcov);
+    s += n_addcov;
+    ss += n_addcov;
 
     /* QTL x interactive covar */
     for(j=0; j<n_intcov; j++) {
-      for(k1=0; k1<n_gen1-1; k1++) { /* QTL1 x intxn */
-	for(k2=0; k2<n_gen2; k2++)
-	  Wts12[k1][k2][i] += param[k1+s]*Intcov[j][i];
+      for(k1=0; k1<n_gen1-1; k1++, ss++, s++) { /* QTL1 x intxn */
+	if(!n_col2drop || !allcol2drop[ss]) {
+	  for(k2=0; k2<n_gen2; k2++)
+	    Wts12[k1][k2][i] += param[s]*Intcov[j][i];
+	}
+	else s--;
       }
-      s += n_gen1-1;
-      for(k2=0; k2<n_gen2-1; k2++) { /* QTL2 x intxn */
-	for(k1=0; k1<n_gen1; k1++)
-	  Wts12[k1][k2][i] += param[k2+s]*Intcov[j][i];
+      for(k2=0; k2<n_gen2-1; k2++, ss++, s++) { /* QTL2 x intxn */
+	if(!n_col2drop || !allcol2drop[ss]) {
+	  for(k1=0; k1<n_gen1; k1++)
+	    Wts12[k1][k2][i] += param[s]*Intcov[j][i];
+	}
+	else s--;
       }
-      s += n_gen2-1;
     }
       
     if(full_model) {
       /* QTL x QTL interaction */
       for(k1=0; k1<n_gen1-1; k1++) 
-	for(k2=0; k2<n_gen2-1; k2++) 
-	  Wts12[k1][k2][i] += param[k1*(n_gen2-1)+k2+s];
-      s += (n_gen1-1)*(n_gen2-1);
+	for(k2=0; k2<n_gen2-1; k2++, ss++, s++) {
+	  if(!n_col2drop || !allcol2drop[ss]) 
+	    Wts12[k1][k2][i] += param[s];
+	  else s--;
+	}
       
       /* QTL x QTL x interactive covar */
       for(j=0; j<n_intcov; j++) {
 	for(k1=0; k1<n_gen1-1; k1++) {
-	  for(k2=0; k2<n_gen2-1; k2++) {
-	    Wts12[k1][k2][i] += param[k1*(n_gen2-1)+k2+s]*Intcov[j][i];
+	  for(k2=0; k2<n_gen2-1; k2++, ss++, s++) {
+	    if(!n_col2drop || !allcol2drop[ss]) 
+	      Wts12[k1][k2][i] += param[s]*Intcov[j][i];
+	    else s--;
 	  }
 	}
-	s += (n_gen1-1)*(n_gen2-1);
       }
-    }
+    } 
     /* done calculating fitted values */
 
     temp = 0.0;
@@ -916,7 +997,8 @@ void scantwo_binary_em_estep(int n_ind, int n_gen1, int n_gen2,
 double scantwo_binary_em_loglik(int n_ind, int n_gen1, int n_gen2, 
 				double ***Probs, double **Addcov, int n_addcov,
 				double **Intcov, int n_intcov, int *pheno,
-				double *param, int full_model)
+				double *param, int full_model,
+				int n_col2drop, int *allcol2drop)
 {
   double *wts, ***Wts;
   double loglik, temp;
@@ -928,7 +1010,8 @@ double scantwo_binary_em_loglik(int n_ind, int n_gen1, int n_gen2,
 
   scantwo_binary_em_estep(n_ind, n_gen1, n_gen2, Probs, Wts, 
 			  Addcov, n_addcov, Intcov, n_intcov, 
-			  pheno, param, full_model, 0);
+			  pheno, param, full_model, 0,
+			  n_col2drop, allcol2drop);
   
   loglik=0.0;
   for(i=0; i<n_ind; i++) {
@@ -940,7 +1023,6 @@ double scantwo_binary_em_loglik(int n_ind, int n_gen1, int n_gen2,
   }
   return(loglik);
 }
-				
 
 /* end of scantwo_binary_em.c */
 
