@@ -24,32 +24,21 @@ library(qtl)
 ##############################
 # Reading
 ##############################
-# Read Karl's format
-karl <- read.cross("karl", genfile="gen.txt",
-                   mapfile="map.txt", phefile="phe.txt")
-karl
-karl2 <- read.cross("karl")
-karl2
-
 # Read CSV format
 csv <- read.cross("csv", , "listeria.csv")
-csv
-csv2 <- read.cross("csv", , "listeria2.csv")
-csv2
+csv2 <- read.cross("csv", , "listeria2.csv", estimate=FALSE)
 
 # Read mapmaker format
 mm <- read.cross("mm", "", "listeria.raw", "listeria.map")
-mm
-mm2 <- read.cross("mm", "", "listeria.raw", "listeria2.map")
-mm2
+mm2 <- read.cross("mm", "", "listeria.raw", "listeria2.map", estimate=FALSE)
 
 ##############################
 # Writing
 ##############################
 # Write in CSV format
-write.cross(karl, "csv", filestem="junk1")
-karl3 <- read.cross("csv", "", "junk1.csv")
-comparecrosses(karl, karl3)
+write.cross(csv, "csv", filestem="junk1")
+csv3 <- read.cross("csv", "", "junk1.csv")
+comparecrosses(csv, csv3)
 
 # Write in mapmaker format
-write.cross(karl, "mm", filestem="junk2")
+write.cross(csv, "mm", filestem="junk2")
